"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildVersionedTx = exports.valueToBase = exports.baseToValue = exports.printSOLBalance = exports.isInValidKeyPair = exports.getBalance = exports.isValidAddress = exports.getPublicKey = exports.createWallet = void 0;
exports.sendSol = sendSol;
exports.getKeyPairFromPrivateKey = getKeyPairFromPrivateKey;
const bs58_1 = require("bs58");
const web3_js_1 = require("@solana/web3.js");
const utils_1 = require("./utils");
const createWallet = () => {
    let keypair = web3_js_1.Keypair.generate();
    let publicKey = keypair.publicKey.toBase58();
    let privateKey = bs58_1.default.encode(keypair.secretKey);
    return { publicKey, privateKey };
};
exports.createWallet = createWallet;
const getPublicKey = (privateKey) => {
    try {
        let keypair = web3_js_1.Keypair.fromSecretKey(bs58_1.default.decode(privateKey));
        let publicKey = keypair.publicKey.toBase58();
        return publicKey;
    }
    catch (error) {
        return null;
    }
};
exports.getPublicKey = getPublicKey;
const isValidAddress = (publicKey) => {
    try {
        const key = new web3_js_1.PublicKey(publicKey);
        return true;
    }
    catch (error) {
        return false;
    }
};
exports.isValidAddress = isValidAddress;
const getBalance = (connection_1, publicKey_1, ...args_1) => __awaiter(void 0, [connection_1, publicKey_1, ...args_1], void 0, function* (connection, publicKey, lamports = false) {
    try {
        const balance = yield connection.getBalance(new web3_js_1.PublicKey(publicKey));
        if (lamports)
            return balance;
        else
            return balance / web3_js_1.LAMPORTS_PER_SOL;
    }
    catch (error) {
        console.log('solana.ts getBalance error :', error);
        return 0;
    }
});
exports.getBalance = getBalance;
function sendSol(connection, sender, receiver, amount) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const transaction = new web3_js_1.Transaction().add(web3_js_1.SystemProgram.transfer({
                fromPubkey: sender.publicKey,
                toPubkey: receiver,
                lamports: amount - 5000,
            }));
            const signature = yield connection.sendTransaction(transaction, [sender]);
            if (signature)
                return true;
            else
                return false;
        }
        catch (error) {
            console.log('send sol error', error);
            return false;
        }
    });
}
function getKeyPairFromPrivateKey(privateKey) {
    return web3_js_1.Keypair.fromSecretKey(bs58_1.default.decode(privateKey));
}
const isInValidKeyPair = (key) => {
    try {
        let keypair = bs58_1.default.encode(key.secretKey);
        if (!keypair) {
            return true;
        }
        else {
            (0, utils_1.log)(`Invalid. ${keypair}`);
            return false;
        }
    }
    catch (error) {
        console.error(error);
        return false;
    }
};
exports.isInValidKeyPair = isInValidKeyPair;
const printSOLBalance = (connection_1, pubKey_1, ...args_1) => __awaiter(void 0, [connection_1, pubKey_1, ...args_1], void 0, function* (connection, pubKey, info = "") {
    const balance = yield connection.getBalance(pubKey);
    console.log(`${info ? info + " " : ""}${pubKey.toBase58()}:`, balance / web3_js_1.LAMPORTS_PER_SOL, `SOL`);
});
exports.printSOLBalance = printSOLBalance;
const baseToValue = (base, decimals) => {
    return base * Math.pow(10, decimals);
};
exports.baseToValue = baseToValue;
const valueToBase = (value, decimals) => {
    return value / Math.pow(10, decimals);
};
exports.valueToBase = valueToBase;
const buildVersionedTx = (connection, payer, tx, commitment) => __awaiter(void 0, void 0, void 0, function* () {
    const blockHash = (yield connection.getLatestBlockhash(commitment))
        .blockhash;
    let messageV0 = new web3_js_1.TransactionMessage({
        payerKey: payer,
        recentBlockhash: blockHash,
        instructions: tx.instructions,
    }).compileToV0Message();
    return new web3_js_1.VersionedTransaction(messageV0);
});
exports.buildVersionedTx = buildVersionedTx;
